import { BaseDbms } from 'meyer';
import Knex from 'knex';

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */

function __awaiter(thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
}

class MssqlDbms extends BaseDbms {
    constructor(config) {
        super();
        this.listMigrations = (tableName) => __awaiter(this, void 0, void 0, function* () {
            // tslint:disable-next-line
            if (!(yield this.knex.schema.hasTable(tableName))) {
                yield new Promise((resolve, reject) => this.knex.schema
                    .createTable(tableName, table => {
                    table
                        .integer('id')
                        .primary()
                        .notNullable();
                    table.text('name').notNullable();
                    table.text('up').notNullable();
                    table.text('down').notNullable();
                    table.text('checksum').notNullable();
                })
                    .then(resolve, reject));
            }
            const migrations = (yield this.knex // tslint:disable-next-line
                .table(tableName)
                .select('id', 'name', 'up', 'down', 'checksum'));
            return migrations;
        });
        this.applyMigration = (tableName, migration, opts) => __awaiter(this, void 0, void 0, function* () {
            yield this.knex.transaction((trx) => __awaiter(this, void 0, void 0, function* () {
                // tslint:disable-next-line
                yield trx.raw(migration.up);
                if (opts.checkEffects) {
                    // tslint:disable-next-line
                    yield trx.raw(migration.down);
                    // tslint:disable-next-line
                    yield trx.raw(migration.up);
                }
                // tslint:disable-next-line
                yield trx.table(tableName).insert(migration);
            }));
        });
        this.revertMigration = (tableName, migration) => __awaiter(this, void 0, void 0, function* () {
            yield this.knex.transaction((trx) => __awaiter(this, void 0, void 0, function* () {
                // tslint:disable-next-line
                yield trx.raw(migration.down);
                // tslint:disable-next-line
                yield trx
                    .table(tableName)
                    .where('id', migration.id)
                    .delete();
            }));
        });
        this.knex = Knex({
            client: 'mssql',
            connection: config,
            pool: { min: 0, max: 1, idleTimeoutMillis: 1 }
        });
    }
}

export default MssqlDbms;
//# sourceMappingURL=mssql.es5.js.map
